@@ -35,7 +35,7 @@ pub(crate) async fn check_added_large_files(
     // `hook.entry` is `check-added-large-files`, set by `pre-commit-hooks`.
     // We don't actually use it, we use it here to parse the arguments.
     // (Parser treats `hook.entry` as a command name.)
-    let args = Args::try_parse_from(hook.entry.parsed()?.iter().chain(&hook.args))?;
+    let args = Args::try_parse_from(hook.entry.resolve(None)?.iter().chain(&hook.args))?;
     let filter = if args.enforce_all {
         FileFilter::NoFilter
     } else {
