@@ -3,31 +3,37 @@ use std::path::PathBuf;
 use std::process::ExitCode;
 
 use clap::builder::styling::{AnsiColor, Effects};
-use clap::builder::{StyledStr, Styles};
+use clap::builder::{ArgPredicate, StyledStr, Styles};
 use clap::{ArgAction, Args, Parser, Subcommand, ValueHint};
 use clap_complete::engine::{ArgValueCompleter, CompletionCandidate};
 
 use constants::env_vars::EnvVars;
+use serde::{Deserialize, Serialize};
 
 use crate::config::{self, CONFIG_FILE, HookType, Stage};
 use crate::workspace::Project;
 
+mod auto_update;
 mod clean;
 mod hook_impl;
 mod install;
 mod list;
-mod reporter;
+pub mod reporter;
 pub mod run;
 mod sample_config;
+#[cfg(feature = "self-update")]
 mod self_update;
 mod validate;
 
+use crate::git::GIT_ROOT;
+pub(crate) use auto_update::auto_update;
 pub(crate) use clean::clean;
 pub(crate) use hook_impl::hook_impl;
 pub(crate) use install::{init_template_dir, install, install_hooks, uninstall};
 pub(crate) use list::list;
 pub(crate) use run::run;
 pub(crate) use sample_config::sample_config;
+#[cfg(feature = "self-update")]
 pub(crate) use self_update::self_update;
 pub(crate) use validate::{validate_configs, validate_manifest};
 
@@ -37,15 +43,8 @@ fn hook_id_completer(current: &std::ffi::OsStr) -> Vec<CompletionCandidate> {
 }
 
 fn get_hook_id_candidates(current: &std::ffi::OsStr) -> anyhow::Result<Vec<CompletionCandidate>> {
-    let output = std::process::Command::new("git")
-        .arg("rev-parse")
-        .arg("--show-toplevel")
-        .output()?;
-
-    let root = String::from_utf8(output.stdout)?.trim().to_string();
-    std::env::set_current_dir(&root).ok();
-
-    let project = Project::from_config_file(None)?;
+    // TODO: find from ancestor directories up to the root of the git repository
+    let project = Project::from_directory(GIT_ROOT.as_ref()?)?;
 
     let hook_ids = project
         .config()
@@ -309,28 +308,49 @@ pub(crate) struct RunExtraArgs {
 #[derive(Debug, Clone, Default, Args)]
 pub(crate) struct RunArgs {
     /// The hook ID to run.
-    #[arg(value_name = "HOOK", value_hint = ValueHint::Other, add = ArgValueCompleter::new(hook_id_completer))]
+    #[arg(
+        value_name = "HOOK",
+        value_hint = ValueHint::Other,
+        add = ArgValueCompleter::new(hook_id_completer)
+    )]
     pub(crate) hook_ids: Vec<String>,
     /// Run on all files in the repo.
     #[arg(short, long, conflicts_with_all = ["files", "from_ref", "to_ref"])]
     pub(crate) all_files: bool,
     /// Specific filenames to run hooks on.
-    #[arg(long, conflicts_with_all = ["all_files", "from_ref", "to_ref"], value_hint = ValueHint::AnyPath)]
+    #[arg(
+        long,
+        conflicts_with_all = ["all_files", "from_ref", "to_ref"],
+        value_hint = ValueHint::AnyPath)
+    ]
     pub(crate) files: Vec<String>,
     /// Run hooks on all files in the specified directories.
     ///
     /// You can specify multiple directories. It can be used in conjunction with `--files`.
-    #[arg(short, long, value_name = "DIR", conflicts_with_all = ["all_files", "from_ref", "to_ref"], value_hint = ValueHint::DirPath)]
+    #[arg(
+        short,
+        long,
+        value_name = "DIR",
+        conflicts_with_all = ["all_files", "from_ref", "to_ref"],
+        value_hint = ValueHint::DirPath
+    )]
     pub(crate) directory: Vec<String>,
-    /// The original ref in a `from_ref...to_ref` diff expression.
+    /// The original ref in a `<from_ref>...<to_ref>` diff expression.
     /// Files changed in this diff will be run through the hooks.
-    #[arg(short = 's', long, alias = "source", requires = "to_ref", value_hint = ValueHint::Other)]
+    #[arg(short = 's', long, alias = "source", value_hint = ValueHint::Other)]
     pub(crate) from_ref: Option<String>,
     /// The destination ref in a `from_ref...to_ref` diff expression.
-    /// Files changed in this diff will be run through the hooks.
-    #[arg(short = 'o', long, alias = "origin", requires = "from_ref", value_hint = ValueHint::Other)]
+    /// Defaults to `HEAD` if `from_ref` is specified.
+    #[arg(
+        short = 'o',
+        long,
+        alias = "origin",
+        requires = "from_ref",
+        value_hint = ValueHint::Other,
+        default_value_if("from_ref", ArgPredicate::IsPresent, "HEAD")
+    )]
     pub(crate) to_ref: Option<String>,
-    /// Run hooks against the last commit (HEAD~1..HEAD).
+    /// Run hooks against the last commit. Equivalent to `--from-ref HEAD~1 --to-ref HEAD`.
     #[arg(long, conflicts_with_all = ["all_files", "files", "directory", "from_ref", "to_ref"])]
     pub(crate) last_commit: bool,
     /// The stage during which the hook is fired.
@@ -344,6 +364,14 @@ pub(crate) struct RunArgs {
     pub(crate) extra: RunExtraArgs,
 }
 
+#[derive(Debug, Clone, Copy, clap::ValueEnum, Default, Serialize, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+pub(crate) enum ListOutputFormat {
+    #[default]
+    Text,
+    Json,
+}
+
 #[derive(Debug, Clone, Default, Args)]
 pub(crate) struct ListArgs {
     #[arg(value_name = "HOOK", value_hint = ValueHint::Other, add = ArgValueCompleter::new(hook_id_completer))]
@@ -354,6 +382,9 @@ pub(crate) struct ListArgs {
     /// Show only hooks that are implemented in the specified language.
     #[arg(long, value_enum)]
     pub(crate) language: Option<config::Language>,
+    /// The output format.
+    #[arg(long, value_enum, default_value_t = ListOutputFormat::Text)]
+    pub(crate) output_format: ListOutputFormat,
 }
 
 #[derive(Debug, Args)]
@@ -384,13 +415,17 @@ pub(crate) struct SampleConfigArgs {
 
 #[derive(Debug, Args)]
 pub(crate) struct AutoUpdateArgs {
-    #[arg(long, default_value_t = true)]
+    /// Update to the bleeding edge of the default branch instead of the latest tagged version.
+    #[arg(long)]
     pub(crate) bleeding_edge: bool,
+    /// Store "frozen" hashes in `rev` instead of tag names.
     #[arg(long)]
     pub(crate) freeze: bool,
+    /// Only update this repository. This option may be specified multiple times.
     #[arg(long)]
-    pub(crate) repo: Option<String>,
-    #[arg(short, long, default_value_t = 1)]
+    pub(crate) repo: Vec<String>,
+    /// Number of threads to use.
+    #[arg(short, long, default_value_t = 3)]
     pub(crate) jobs: usize,
 }
 
