@@ -8,11 +8,8 @@ use std::path::PathBuf;
 use std::rc::Rc;
 use std::sync::Arc;
 
-use anstream::ColorChoice;
 use anyhow::{Context, Result};
-use futures::StreamExt;
-use futures::stream::FuturesUnordered;
-use indoc::indoc;
+use futures::stream::{FuturesUnordered, StreamExt};
 use owo_colors::{OwoColorize, Style};
 use rand::SeedableRng;
 use rand::prelude::{SliceRandom, StdRng};
@@ -28,12 +25,13 @@ use crate::cli::run::keeper::WorkTreeKeeper;
 use crate::cli::run::{CollectOptions, FileFilter, collect_files};
 use crate::cli::{ExitStatus, RunExtraArgs};
 use crate::config::{Language, Stage};
-use crate::fs::Simplified;
-use crate::git;
+use crate::fs::{CWD, Simplified};
 use crate::hook::{Hook, InstalledHook};
 use crate::printer::{Printer, Stdout};
-use crate::store::Store;
+use crate::run::USE_COLOR;
+use crate::store::{STORE, Store};
 use crate::workspace::Project;
+use crate::{git, warn_user};
 
 enum HookToRun {
     Skipped(Arc<Hook>),
@@ -87,57 +85,58 @@ pub(crate) async fn run(
     if should_stash && git::has_unmerged_paths().await? {
         writeln!(
             printer.stderr(),
-            "You have unmerged paths. Resolve them before running prek."
+            "{}: You have unmerged paths. Resolve them before running prek",
+            "error".red().bold(),
         )?;
         return Ok(ExitStatus::Failure);
     }
 
-    let config_file = Project::find_config_file(config)?;
-    if should_stash && git::file_not_staged(&config_file).await? {
+    let mut project = Project::from_config_file_or_directory(config, &CWD)?;
+    if should_stash && git::file_not_staged(project.config_file()).await? {
         writeln!(
             printer.stderr(),
-            indoc!(
-                "Your pre-commit configuration file is not staged.
-                Run `git add {}` to fix this."
-            ),
-            &config_file.user_display()
+            "{}: prek configuration file is not staged, run `{}` to stage it",
+            "error".red().bold(),
+            format!("git add {}", project.config_file().user_display()).cyan(),
         )?;
         return Ok(ExitStatus::Failure);
     }
 
-    let mut project = Project::new(config_file)?;
-    let store = Store::from_settings()?.init()?;
-
+    let store = STORE.as_ref()?;
     let reporter = HookInitReporter::from(printer);
 
     let lock = store.lock_async().await?;
-    let hooks = project.init_hooks(&store, Some(&reporter)).await?;
 
     let hook_ids = hook_ids.into_iter().collect::<BTreeSet<_>>();
-    let hooks: Vec<_> = hooks
+
+    let hooks = project.init_hooks(store, Some(&reporter)).await?;
+    let filtered_hooks: Vec<_> = hooks
         .into_iter()
         .filter(|h| hook_ids.is_empty() || hook_ids.contains(&h.id) || hook_ids.contains(&h.alias))
         .filter(|h| h.stages.contains(hook_stage))
         .collect();
 
-    if hooks.is_empty() {
+    if filtered_hooks.is_empty() {
         if hook_ids.is_empty() {
             writeln!(
                 printer.stderr(),
-                "No hooks found for stage `{}`",
+                "{}: No hooks found for stage `{}`",
+                "error".red().bold(),
                 hook_stage.cyan()
             )?;
         } else if hook_ids.len() == 1 {
             writeln!(
                 printer.stderr(),
-                "No hook found with id `{}` in stage `{}`",
+                "{}: No hook found with id `{}` in stage `{}`",
+                "error".red().bold(),
                 hook_ids.iter().next().unwrap().cyan(),
                 hook_stage.cyan()
             )?;
         } else {
             writeln!(
                 printer.stderr(),
-                "No hooks found with ids {} in stage `{}`",
+                "{}: No hooks found with ids {} in stage `{}`",
+                "error".red().bold(),
                 hook_ids
                     .iter()
                     .map(|id| format!("`{}`", id.cyan()))
@@ -147,15 +146,38 @@ pub(crate) async fn run(
             )?;
         }
         return Ok(ExitStatus::Failure);
+    } else if !hook_ids.is_empty() {
+        // Warn about hook IDs that don't match any hooks
+        let matched_ids: BTreeSet<String> = filtered_hooks
+            .iter()
+            .flat_map(|h| [h.id.clone(), h.alias.clone()])
+            .collect();
+
+        let unmatched_ids: Vec<&String> = hook_ids
+            .iter()
+            .filter(|id| !matched_ids.contains(*id))
+            .collect();
+
+        if !unmatched_ids.is_empty() {
+            warn_user!(
+                "Ignored non-existent hook ID{}: {}",
+                if unmatched_ids.len() > 1 { "s" } else { "" },
+                unmatched_ids
+                    .iter()
+                    .map(|id| format!("`{}`", id.yellow()))
+                    .collect::<Vec<_>>()
+                    .join(", ")
+            );
+        }
     }
 
     let skips = get_skips();
-    let skips = hooks
+    let skips = filtered_hooks
         .iter()
         .filter(|h| skips.contains(&h.id) || skips.contains(&h.alias))
         .map(|h| h.idx)
         .collect::<FxHashSet<_>>();
-    let to_run = hooks
+    let to_run = filtered_hooks
         .iter()
         .filter(|h| !skips.contains(&h.idx))
         .cloned()
@@ -166,12 +188,12 @@ pub(crate) async fn run(
         to_run.iter().map(|h| &h.id).collect::<Vec<_>>()
     );
     let reporter = HookInstallReporter::from(printer);
-    let mut installed_hooks = install_hooks(to_run, &store, &reporter).await?;
+    let mut installed_hooks = install_hooks(to_run, store, &reporter).await?;
 
     // Release the store lock.
     drop(lock);
 
-    let hooks = hooks
+    let hooks = filtered_hooks
         .into_iter()
         .map(|h| {
             if skips.contains(&h.idx) {
@@ -190,7 +212,7 @@ pub(crate) async fn run(
     // Clear any unstaged changes from the git working directory.
     let mut _guard = None;
     if should_stash {
-        _guard = Some(WorkTreeKeeper::clean(&store).await?);
+        _guard = Some(WorkTreeKeeper::clean(store).await?);
     }
 
     set_env_vars(from_ref.as_ref(), to_ref.as_ref(), &extra_args);
@@ -208,15 +230,15 @@ pub(crate) async fn run(
 
     let filter = FileFilter::new(
         &filenames,
-        project.config().files.as_deref(),
-        project.config().exclude.as_deref(),
-    )?;
+        project.config().files.as_ref(),
+        project.config().exclude.as_ref(),
+    );
     trace!("Files after filtered: {}", filter.len());
 
     run_hooks(
         &hooks,
         &filter,
-        &store,
+        store,
         project.config().fail_fast.unwrap_or(false),
         show_diff_on_failure,
         verbose,
@@ -351,11 +373,9 @@ pub async fn install_hooks(
                     let hook = Arc::new(hook);
                     debug!("No matching environment found for hook `{hook}`, installing...");
 
-                    let progress = reporter.on_install_start(&hook);
-
                     let installed_hook = hook
                         .language
-                        .install(hook.clone(), store)
+                        .install(hook.clone(), store, reporter)
                         .await
                         .context(format!("Failed to install hook `{hook}`"))?;
 
@@ -374,8 +394,6 @@ pub async fn install_hooks(
                     }
 
                     newly_installed.push(installed_hook);
-
-                    reporter.on_install_complete(progress);
                 }
 
                 // Add newly installed hooks to the list.
@@ -552,10 +570,10 @@ async fn run_hooks(
 
     if !success && show_diff_on_failure {
         writeln!(printer.stdout(), "All changes made by hooks:")?;
-        let color = match ColorChoice::global() {
-            ColorChoice::Auto => "--color=auto",
-            ColorChoice::Always | ColorChoice::AlwaysAnsi => "--color=always",
-            ColorChoice::Never => "--color=never",
+        let color = if *USE_COLOR {
+            "--color=always"
+        } else {
+            "--color=never"
         };
         git::git_cmd("git diff")?
             .arg("--no-pager")
@@ -599,7 +617,7 @@ async fn run_hook(
         HookToRun::ToRun(hook) => hook,
     };
 
-    let mut filenames = filter.for_hook(hook)?;
+    let mut filenames = filter.for_hook(hook);
 
     if filenames.is_empty() && !hook.always_run {
         printer.write_skipped(
