@@ -1,20 +1,58 @@
 use std::collections::HashMap;
 use std::fmt::Display;
-use std::ops::RangeInclusive;
+use std::ops::{Deref, RangeInclusive};
 use std::path::Path;
 use std::str::FromStr;
+use std::sync::OnceLock;
 
 use anyhow::Result;
-use fancy_regex as regex;
-use serde::{Deserialize, Deserializer, Serialize};
-use url::Url;
+use fancy_regex::{self as regex, Regex};
+use serde::{Deserialize, Deserializer, Serialize, Serializer};
 
 use crate::fs::Simplified;
+use crate::version;
+use crate::warn_user;
 
 pub const CONFIG_FILE: &str = ".pre-commit-config.yaml";
-pub const ALTER_CONFIG_FILE: &str = ".pre-commit-config.yml";
 pub const MANIFEST_FILE: &str = ".pre-commit-hooks.yaml";
 
+#[derive(Clone)]
+pub struct SerdeRegex(Regex);
+
+impl Deref for SerdeRegex {
+    type Target = Regex;
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
+impl std::fmt::Debug for SerdeRegex {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_tuple("SerdeRegex").field(&self.0.as_str()).finish()
+    }
+}
+
+impl<'de> Deserialize<'de> for SerdeRegex {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        let s = String::deserialize(deserializer)?;
+        Regex::new(&s)
+            .map(SerdeRegex)
+            .map_err(serde::de::Error::custom)
+    }
+}
+
+impl Serialize for SerdeRegex {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serializer.serialize_str(self.0.as_str())
+    }
+}
+
 #[derive(Debug, Copy, Clone, PartialEq, Eq, Hash, Deserialize, Serialize, clap::ValueEnum)]
 #[serde(rename_all = "snake_case")]
 pub enum Language {
@@ -128,7 +166,9 @@ impl Display for HookType {
     }
 }
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq, Default, Hash, Deserialize, clap::ValueEnum)]
+#[derive(
+    Debug, Clone, Copy, PartialEq, Eq, Default, Hash, Deserialize, Serialize, clap::ValueEnum,
+)]
 #[serde(rename_all = "kebab-case")]
 pub enum Stage {
     Manual,
@@ -203,15 +243,38 @@ impl Stage {
     }
 }
 
-// TODO: warn unexpected keys
+fn deserialize_minimum_version<'de, D>(deserializer: D) -> Result<Option<String>, D::Error>
+where
+    D: Deserializer<'de>,
+{
+    let s = String::deserialize(deserializer)?;
+    if s.is_empty() {
+        return Ok(None);
+    }
+
+    let version = s
+        .parse::<semver::Version>()
+        .map_err(serde::de::Error::custom)?;
+    let cur_version = version::version()
+        .version
+        .parse::<semver::Version>()
+        .expect("Invalid prek version");
+    if version > cur_version {
+        return Err(serde::de::Error::custom(format!(
+            "Required minimum prek version `{version}` is greater than current version `{cur_version}`. Please consider updating prek.",
+        )));
+    }
+
+    Ok(Some(s))
+}
+
 // TODO: warn deprecated stage
 // TODO: warn sensible regex
-// TODO: check minimum_pre_commit_version
 #[derive(Debug, Clone, Deserialize)]
 #[serde(rename_all = "snake_case")]
 pub struct Config {
     pub repos: Vec<Repo>,
-    /// A list of --hook-types which will be used by default when running pre-commit install.
+    /// A list of `--hook-types` which will be used by default when running `prek install`.
     /// Default is `[pre-commit]`.
     pub default_install_hook_types: Option<Vec<HookType>>,
     /// A mapping from language to the default `language_version`.
@@ -220,13 +283,15 @@ pub struct Config {
     /// Default to all stages.
     pub default_stages: Option<Vec<Stage>>,
     /// Global file include pattern.
-    pub files: Option<String>,
+    pub files: Option<SerdeRegex>,
     /// Global file exclude pattern.
-    pub exclude: Option<String>,
-    /// Set to true to have pre-commit stop running hooks after the first failure.
+    pub exclude: Option<SerdeRegex>,
+    /// Set to true to have prek stop running hooks after the first failure.
     /// Default is false.
     pub fail_fast: Option<bool>,
-    pub minimum_pre_commit_version: Option<String>,
+    /// The minimum version of prek required to run this configuration.
+    #[serde(deserialize_with = "deserialize_minimum_version", default)]
+    pub minimum_prek_version: Option<String>,
     /// Configuration for pre-commit.ci service.
     pub ci: Option<HashMap<String, serde_yaml::Value>>,
 }
@@ -236,7 +301,7 @@ pub struct Config {
 pub enum RepoLocation {
     Local,
     Meta,
-    Remote(Url),
+    Remote(String),
 }
 
 impl FromStr for RepoLocation {
@@ -246,7 +311,7 @@ impl FromStr for RepoLocation {
         match s {
             "local" => Ok(RepoLocation::Local),
             "meta" => Ok(RepoLocation::Meta),
-            _ => Url::parse(s).map(RepoLocation::Remote),
+            _ => Ok(RepoLocation::Remote(s.to_string())),
         }
     }
 }
@@ -283,10 +348,10 @@ pub struct HookOptions {
     /// Not documented in the official docs.
     pub alias: Option<String>,
     /// The pattern of files to run on.
-    pub files: Option<String>,
+    pub files: Option<SerdeRegex>,
     /// Exclude files that were matched by `files`.
     /// Default is `$^`, which matches nothing.
-    pub exclude: Option<String>,
+    pub exclude: Option<SerdeRegex>,
     /// List of file types to run on (AND).
     /// Default is `[file]`, which matches all files.
     pub types: Option<Vec<String>>,
@@ -327,7 +392,9 @@ pub struct HookOptions {
     /// Print the output of the hook even if it passes.
     /// Default is false.
     pub verbose: Option<bool>,
-    pub minimum_pre_commit_version: Option<String>,
+    /// The minimum version of prek required to run this hook.
+    #[serde(deserialize_with = "deserialize_minimum_version", default)]
+    pub minimum_prek_version: Option<String>,
 }
 
 impl HookOptions {
@@ -360,7 +427,7 @@ impl HookOptions {
             require_serial,
             stages,
             verbose,
-            minimum_pre_commit_version,
+            minimum_prek_version,
         );
     }
 }
@@ -453,11 +520,11 @@ impl<'de> Deserialize<'de> for MetaHook {
                 language: Language::System,
                 entry: String::new(),
                 options: HookOptions {
-                    files: Some(format!(
-                        "^{}|{}$",
-                        regex::escape(CONFIG_FILE),
-                        regex::escape(ALTER_CONFIG_FILE)
-                    )),
+                    files: Some(
+                        Regex::new(&format!("^{}$", regex::escape(CONFIG_FILE),))
+                            .map(SerdeRegex)
+                            .unwrap(),
+                    ),
                     ..Default::default()
                 },
             },
@@ -467,11 +534,11 @@ impl<'de> Deserialize<'de> for MetaHook {
                 language: Language::System,
                 entry: String::new(),
                 options: HookOptions {
-                    files: Some(format!(
-                        "^{}|{}$",
-                        regex::escape(CONFIG_FILE),
-                        regex::escape(ALTER_CONFIG_FILE)
-                    )),
+                    files: Some(
+                        Regex::new(&format!("^{}$", regex::escape(CONFIG_FILE),))
+                            .map(SerdeRegex)
+                            .unwrap(),
+                    ),
                     ..Default::default()
                 },
             },
@@ -501,7 +568,7 @@ impl From<MetaHook> for ManifestHook {
 
 #[derive(Debug, Clone, Serialize)]
 pub struct RemoteRepo {
-    pub repo: Url,
+    pub repo: String,
     pub rev: String,
     #[serde(skip)]
     pub hooks: Vec<RemoteHook>,
@@ -574,6 +641,7 @@ impl<'de> Deserialize<'de> for Repo {
         match repo {
             RepoLocation::Remote(url) => {
                 #[derive(Deserialize)]
+                #[serde(deny_unknown_fields)]
                 struct _RemoteRepo {
                     rev: String,
                     hooks: Vec<RemoteHook>,
@@ -620,7 +688,7 @@ pub struct ManifestHook {
     pub name: String,
     /// The command to run. It can contain arguments that will not be overridden.
     pub entry: String,
-    /// The language of the hook. Tells pre-commit how to install and run the hook.
+    /// The language of the hook. Tells prek how to install and run the hook.
     pub language: Language,
     #[serde(flatten)]
     pub options: HookOptions,
@@ -643,9 +711,6 @@ pub enum Error {
 
     #[error("Failed to parse `{0}`")]
     Yaml(String, #[source] serde_yaml::Error),
-
-    #[error("Invalid repo URL: {0}")]
-    RepoUrl(#[from] url::ParseError),
 }
 
 /// Read the configuration file from the given path.
@@ -657,8 +722,68 @@ pub fn read_config(path: &Path) -> Result<Config, Error> {
         }
         Err(e) => return Err(e.into()),
     };
-    let config = serde_yaml::from_str(&content)
-        .map_err(|e| Error::Yaml(path.user_display().to_string(), e))?;
+
+    let expected_unused = ["minimum_pre_commit_version"];
+
+    let deserializer = serde_yaml::Deserializer::from_str(&content);
+    let mut unused = Vec::new();
+    let config: Config = serde_ignored::deserialize(deserializer, |path| {
+        let key = path.to_string();
+        if !expected_unused.contains(&key.as_str()) {
+            unused.push(key);
+        }
+    })
+    .map_err(|e| Error::Yaml(path.user_display().to_string(), e))?;
+
+    if !unused.is_empty() {
+        warn_user!(
+            "Ignored unexpected keys in `{}`: {}",
+            path.display().cyan(),
+            unused
+                .into_iter()
+                .map(|key| format!("`{}`", key.yellow()))
+                .collect::<Vec<_>>()
+                .join(", ")
+        );
+    }
+
+    // Check for mutable revs and warn the user.
+    let repos_has_mutable_rev = config
+        .repos
+        .iter()
+        .filter_map(|repo| {
+            if let Repo::Remote(repo) = repo {
+                let rev = &repo.rev;
+                // A rev is considered mutable if it doesn't contain a '.' (like a version)
+                // and is not a hexadecimal string (like a commit SHA).
+                if !rev.contains('.') && !looks_like_sha(rev) {
+                    return Some(repo);
+                }
+            }
+            None
+        })
+        .collect::<Vec<_>>();
+    if !repos_has_mutable_rev.is_empty() {
+        let msg = repos_has_mutable_rev
+            .iter()
+            .map(|repo| format!("{}: {}", repo.repo.cyan(), repo.rev.yellow()))
+            .collect::<Vec<_>>()
+            .join("\n");
+
+        warn_user!(
+            "{}",
+            indoc::formatdoc! { r#"
+            The following repos have mutable `rev` fields (moving tag / branch):
+            {}
+            Mutable references are never updated after first install and are not supported.
+            See https://pre-commit.com/#using-the-latest-version-for-a-repository for more details.
+            Hint: `prek autoupdate` often fixes this",
+            "#,
+            msg
+            }
+        );
+    }
+
     Ok(config)
 }
 
@@ -670,6 +795,13 @@ pub fn read_manifest(path: &Path) -> Result<Manifest, Error> {
     Ok(manifest)
 }
 
+/// Check if a string looks like a git SHA
+fn looks_like_sha(s: &str) -> bool {
+    static SHA_RE: OnceLock<Regex> = OnceLock::new();
+    let re = SHA_RE.get_or_init(|| Regex::new(r"^[a-fA-F0-9]+$").unwrap());
+    re.is_match(s).unwrap_or(false)
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -717,7 +849,7 @@ mod tests {
                                         require_serial: None,
                                         stages: None,
                                         verbose: None,
-                                        minimum_pre_commit_version: None,
+                                        minimum_prek_version: None,
                                     },
                                 },
                             ],
@@ -730,7 +862,7 @@ mod tests {
                 files: None,
                 exclude: None,
                 fail_fast: None,
-                minimum_pre_commit_version: None,
+                minimum_prek_version: None,
                 ci: None,
             },
         )
@@ -768,21 +900,7 @@ mod tests {
                 repos: [
                     Remote(
                         RemoteRepo {
-                            repo: Url {
-                                scheme: "https",
-                                cannot_be_a_base: false,
-                                username: "",
-                                password: None,
-                                host: Some(
-                                    Domain(
-                                        "github.com",
-                                    ),
-                                ),
-                                port: None,
-                                path: "/crate-ci/typos",
-                                query: None,
-                                fragment: None,
-                            },
+                            repo: "https://github.com/crate-ci/typos",
                             rev: "v1.0.0",
                             hooks: [
                                 RemoteHook {
@@ -808,7 +926,7 @@ mod tests {
                                         require_serial: None,
                                         stages: None,
                                         verbose: None,
-                                        minimum_pre_commit_version: None,
+                                        minimum_prek_version: None,
                                     },
                                 },
                             ],
@@ -821,7 +939,7 @@ mod tests {
                 files: None,
                 exclude: None,
                 fail_fast: None,
-                minimum_pre_commit_version: None,
+                minimum_prek_version: None,
                 ci: None,
             },
         )
@@ -917,7 +1035,7 @@ mod tests {
                                         require_serial: None,
                                         stages: None,
                                         verbose: None,
-                                        minimum_pre_commit_version: None,
+                                        minimum_prek_version: None,
                                     },
                                 },
                             ],
@@ -930,7 +1048,7 @@ mod tests {
                 files: None,
                 exclude: None,
                 fail_fast: None,
-                minimum_pre_commit_version: None,
+                minimum_prek_version: None,
                 ci: None,
             },
         )
@@ -1025,7 +1143,9 @@ mod tests {
                                         options: HookOptions {
                                             alias: None,
                                             files: Some(
-                                                "^\\.pre-commit-config\\.yaml|\\.pre-commit-config\\.yml$",
+                                                SerdeRegex(
+                                                    "^\\.pre-commit-config\\.yaml$",
+                                                ),
                                             ),
                                             exclude: None,
                                             types: None,
@@ -1042,7 +1162,7 @@ mod tests {
                                             require_serial: None,
                                             stages: None,
                                             verbose: None,
-                                            minimum_pre_commit_version: None,
+                                            minimum_prek_version: None,
                                         },
                                     },
                                 ),
@@ -1055,7 +1175,9 @@ mod tests {
                                         options: HookOptions {
                                             alias: None,
                                             files: Some(
-                                                "^\\.pre-commit-config\\.yaml|\\.pre-commit-config\\.yml$",
+                                                SerdeRegex(
+                                                    "^\\.pre-commit-config\\.yaml$",
+                                                ),
                                             ),
                                             exclude: None,
                                             types: None,
@@ -1072,7 +1194,7 @@ mod tests {
                                             require_serial: None,
                                             stages: None,
                                             verbose: None,
-                                            minimum_pre_commit_version: None,
+                                            minimum_prek_version: None,
                                         },
                                     },
                                 ),
@@ -1102,7 +1224,7 @@ mod tests {
                                             verbose: Some(
                                                 true,
                                             ),
-                                            minimum_pre_commit_version: None,
+                                            minimum_prek_version: None,
                                         },
                                     },
                                 ),
@@ -1116,7 +1238,7 @@ mod tests {
                 files: None,
                 exclude: None,
                 fail_fast: None,
-                minimum_pre_commit_version: None,
+                minimum_prek_version: None,
                 ci: None,
             },
         )
@@ -1178,7 +1300,7 @@ mod tests {
                                         require_serial: None,
                                         stages: None,
                                         verbose: None,
-                                        minimum_pre_commit_version: None,
+                                        minimum_prek_version: None,
                                     },
                                 },
                                 ManifestHook {
@@ -1206,7 +1328,7 @@ mod tests {
                                         require_serial: None,
                                         stages: None,
                                         verbose: None,
-                                        minimum_pre_commit_version: None,
+                                        minimum_prek_version: None,
                                     },
                                 },
                                 ManifestHook {
@@ -1234,7 +1356,7 @@ mod tests {
                                         require_serial: None,
                                         stages: None,
                                         verbose: None,
-                                        minimum_pre_commit_version: None,
+                                        minimum_prek_version: None,
                                     },
                                 },
                             ],
@@ -1247,7 +1369,7 @@ mod tests {
                 files: None,
                 exclude: None,
                 fail_fast: None,
-                minimum_pre_commit_version: None,
+                minimum_prek_version: None,
                 ci: None,
             },
         )
@@ -1267,4 +1389,66 @@ mod tests {
         insta::assert_debug_snapshot!(manifest);
         Ok(())
     }
+
+    #[test]
+    fn test_minimum_prek_version() {
+        // Test that missing minimum_prek_version field doesn't cause an error
+        let yaml = indoc::indoc! {r"
+            repos:
+              - repo: local
+                hooks:
+                  - id: test-hook
+                    name: Test Hook
+                    entry: echo test
+                    language: system
+        "};
+        let result = serde_yaml::from_str::<Config>(yaml);
+        assert!(result.is_ok());
+        let config = result.unwrap();
+        assert!(config.minimum_prek_version.is_none());
+
+        // Test that empty minimum_prek_version field is treated as None
+        let yaml = indoc::indoc! {r"
+            repos:
+              - repo: local
+                hooks:
+                  - id: test-hook
+                    name: Test Hook
+                    entry: echo test
+                    language: system
+            minimum_prek_version: ''
+        "};
+        let result = serde_yaml::from_str::<Config>(yaml);
+        assert!(result.is_ok());
+        let config = result.unwrap();
+        assert!(config.minimum_prek_version.is_none());
+
+        // Test that valid minimum_prek_version field works in top-level config
+        let yaml = indoc::indoc! {r"
+            repos:
+              - repo: local
+                hooks:
+                  - id: test-hook
+                    name: Test Hook
+                    entry: echo test
+                    language: system
+            minimum_prek_version: '10.0.0'
+        "};
+        let result = serde_yaml::from_str::<Config>(yaml);
+        assert!(result.is_err());
+
+        // Test that valid minimum_prek_version field works in hook config
+        let yaml = indoc::indoc! {r"
+            repos:
+              - repo: local
+                hooks:
+                  - id: test-hook
+                    name: Test Hook
+                    entry: echo test
+                    language: system
+                    minimum_prek_version: '10.0.0'
+        "};
+        let result = serde_yaml::from_str::<Config>(yaml);
+        assert!(result.is_err());
+    }
 }
