@@ -1,3 +1,4 @@
+use std::ffi::OsStr;
 use std::fmt::{Display, Formatter};
 use std::hash::{Hash, Hasher};
 use std::ops::Deref;
@@ -10,27 +11,34 @@ use rand::Rng;
 use rustc_hash::{FxBuildHasher, FxHashMap, FxHashSet};
 use serde::{Deserialize, Serialize};
 use thiserror::Error;
-use tracing::error;
-use url::Url;
+use tracing::{error, warn};
 
 use crate::config::{
     self, Config, HookOptions, Language, LocalHook, MANIFEST_FILE, ManifestHook, MetaHook,
-    RemoteHook, Stage, read_manifest,
+    RemoteHook, SerdeRegex, Stage, read_manifest,
 };
 use crate::languages::version::LanguageRequest;
+use crate::languages::{extract_metadata_from_entry, resolve_command};
 use crate::store::Store;
 
 #[derive(Error, Debug)]
 pub(crate) enum Error {
     #[error(transparent)]
-    InvalidConfig(#[from] config::Error),
+    Config(#[from] config::Error),
 
     #[error("Hook `{hook}` is invalid")]
-    InvalidHook {
+    Hook {
         hook: String,
         #[source]
         error: anyhow::Error,
     },
+
+    #[error("Failed to read manifest of `{repo}`")]
+    Manifest {
+        repo: String,
+        #[source]
+        error: config::Error,
+    },
 }
 
 #[derive(Debug, Clone)]
@@ -38,7 +46,7 @@ pub(crate) enum Repo {
     Remote {
         /// Path to the cloned repo.
         path: PathBuf,
-        url: Url,
+        url: String,
         rev: String,
         hooks: Vec<ManifestHook>,
     },
@@ -52,8 +60,11 @@ pub(crate) enum Repo {
 
 impl Repo {
     /// Load the remote repo manifest from the path.
-    pub(crate) fn remote(url: Url, rev: String, path: PathBuf) -> Result<Self, Error> {
-        let manifest = read_manifest(&path.join(MANIFEST_FILE))?;
+    pub(crate) fn remote(url: String, rev: String, path: PathBuf) -> Result<Self, Error> {
+        let manifest = read_manifest(&path.join(MANIFEST_FILE)).map_err(|e| Error::Manifest {
+            repo: url.to_string(),
+            error: e,
+        })?;
         let hooks = manifest.hooks;
 
         Ok(Self::Remote {
@@ -181,7 +192,7 @@ impl HookBuilder {
             .map_or(&[][..], |deps| deps.as_slice());
 
         if !language.supports_dependency() && !additional_dependencies.is_empty() {
-            return Err(Error::InvalidHook {
+            return Err(Error::Hook {
                 hook: self.config.id.clone(),
                 error: anyhow::anyhow!(
                     "Hook specified `additional_dependencies` `{}` but the language `{}` does not support installing dependencies for now",
@@ -195,7 +206,7 @@ impl HookBuilder {
             if let Some(language_version) = language_version
                 && language_version != "default"
             {
-                return Err(Error::InvalidHook {
+                return Err(Error::Hook {
                     hook: self.config.id.clone(),
                     error: anyhow::anyhow!(
                         "Hook specified `language_version` `{}` but the language `{}` does not install an environment",
@@ -206,7 +217,7 @@ impl HookBuilder {
             }
 
             if !additional_dependencies.is_empty() {
-                return Err(Error::InvalidHook {
+                return Err(Error::Hook {
                     hook: self.config.id.clone(),
                     error: anyhow::anyhow!(
                         "Hook specified `additional_dependencies` `{}` but the language `{}` does not install an environment",
@@ -221,14 +232,14 @@ impl HookBuilder {
     }
 
     /// Build the hook.
-    pub(crate) fn build(mut self) -> Result<Hook, Error> {
+    pub(crate) async fn build(mut self) -> Result<Hook, Error> {
         self.check()?;
         self.fill_in_defaults();
 
         let options = self.config.options;
         let language_version = options.language_version.expect("language_version not set");
         let language_request = LanguageRequest::parse(self.config.language, &language_version)
-            .map_err(|e| Error::InvalidHook {
+            .map_err(|e| Error::Hook {
                 hook: self.config.id.clone(),
                 error: anyhow::anyhow!(e),
             })?;
@@ -253,11 +264,13 @@ impl HookBuilder {
             None => Stages::All,
         };
 
-        Ok(Hook {
+        let mut hook = Hook {
             entry,
+            stages,
             language_request,
             additional_dependencies,
             dependencies: OnceLock::new(),
+
             repo: self.repo,
             idx: self.idx,
             id: self.config.id,
@@ -276,10 +289,15 @@ impl HookBuilder {
             description: options.description,
             log_file: options.log_file,
             require_serial: options.require_serial.expect("require_serial not set"),
-            stages,
             verbose: options.verbose.expect("verbose not set"),
-            minimum_pre_commit_version: options.minimum_pre_commit_version,
-        })
+            minimum_prek_version: options.minimum_prek_version,
+        };
+
+        if let Err(err) = extract_metadata_from_entry(&mut hook).await {
+            warn!("Failed to extract metadata from entry for hook `{hook}`: {err}");
+        }
+
+        Ok(hook)
     }
 }
 
@@ -325,14 +343,23 @@ impl Entry {
         Self { hook, entry }
     }
 
-    pub(crate) fn parsed(&self) -> Result<Vec<String>, Error> {
-        shlex::split(&self.entry).ok_or_else(|| Error::InvalidHook {
+    /// Split the entry and resolve the command by parsing its shebang.
+    pub(crate) fn resolve(&self, env_path: Option<&OsStr>) -> Result<Vec<String>, Error> {
+        let split = self.split()?;
+
+        Ok(resolve_command(split, env_path))
+    }
+
+    /// Split the entry into a list of commands.
+    pub(crate) fn split(&self) -> Result<Vec<String>, Error> {
+        shlex::split(&self.entry).ok_or_else(|| Error::Hook {
             hook: self.hook.clone(),
             error: anyhow::anyhow!("Failed to parse entry `{}` as commands", &self.entry),
         })
     }
 
-    pub(crate) fn entry(&self) -> &str {
+    /// Get the original entry string.
+    pub(crate) fn raw(&self) -> &str {
         &self.entry
     }
 }
@@ -351,8 +378,8 @@ pub(crate) struct Hook {
     pub entry: Entry,
     pub language: Language,
     pub alias: String,
-    pub files: Option<String>,
-    pub exclude: Option<String>,
+    pub files: Option<SerdeRegex>,
+    pub exclude: Option<SerdeRegex>,
     pub types: Vec<String>,
     pub types_or: Vec<String>,
     pub exclude_types: Vec<String>,
@@ -367,7 +394,7 @@ pub(crate) struct Hook {
     pub require_serial: bool,
     pub stages: Stages,
     pub verbose: bool,
-    pub minimum_pre_commit_version: Option<String>,
+    pub minimum_prek_version: Option<String>,
 }
 
 impl Display for Hook {
