@@ -2,6 +2,7 @@ use std::sync::Arc;
 
 use anyhow::Result;
 
+use crate::cli::reporter::HookInstallReporter;
 use crate::hook::{Hook, InstalledHook};
 use crate::languages::LanguageImpl;
 use crate::store::Store;
@@ -10,7 +11,12 @@ use crate::store::Store;
 pub(crate) struct Fail;
 
 impl LanguageImpl for Fail {
-    async fn install(&self, hook: Arc<Hook>, _store: &Store) -> Result<InstalledHook> {
+    async fn install(
+        &self,
+        hook: Arc<Hook>,
+        _store: &Store,
+        _reporter: &HookInstallReporter,
+    ) -> Result<InstalledHook> {
         Ok(InstalledHook::NoNeedInstall(hook))
     }
 
@@ -24,7 +30,7 @@ impl LanguageImpl for Fail {
         filenames: &[&String],
         _store: &Store,
     ) -> Result<(i32, Vec<u8>)> {
-        let mut out = shlex::try_join(hook.entry.parsed()?.iter().map(std::ops::Deref::deref))
+        let mut out = shlex::try_join(hook.entry.resolve(None)?.iter().map(std::ops::Deref::deref))
             .expect("Failed to join `entry` as command")
             .into_bytes();
         out.extend(b"\n\n");
