@@ -6,6 +6,7 @@ use anyhow::Context;
 
 use constants::env_vars::EnvVars;
 
+use crate::cli::reporter::HookInstallReporter;
 use crate::hook::{Hook, InstallInfo, InstalledHook};
 use crate::languages::LanguageImpl;
 use crate::languages::golang::GoRequest;
@@ -19,7 +20,14 @@ use crate::store::{CacheBucket, Store};
 pub(crate) struct Golang;
 
 impl LanguageImpl for Golang {
-    async fn install(&self, hook: Arc<Hook>, store: &Store) -> anyhow::Result<InstalledHook> {
+    async fn install(
+        &self,
+        hook: Arc<Hook>,
+        store: &Store,
+        reporter: &HookInstallReporter,
+    ) -> anyhow::Result<InstalledHook> {
+        let progress = reporter.on_install_start(&hook);
+
         // 1. Install Go
         let go_dir = store.tools_path(crate::store::ToolBucket::Go);
         let installer = GoInstaller::new(go_dir);
@@ -29,7 +37,10 @@ impl LanguageImpl for Golang {
             LanguageRequest::Golang(version) => version,
             _ => unreachable!(),
         };
-        let go = installer.install(version).await?;
+        let go = installer
+            .install(version)
+            .await
+            .context("Failed to install go")?;
 
         let mut info = InstallInfo::new(
             hook.language,
@@ -78,14 +89,22 @@ impl LanguageImpl for Golang {
             go_install_cmd()
                 .arg("./...")
                 .current_dir(repo)
+                .remove_git_env()
                 .check(true)
                 .output()
                 .await?;
         }
         for dep in &hook.additional_dependencies {
-            go_install_cmd().arg(dep).check(true).output().await?;
+            go_install_cmd()
+                .arg(dep)
+                .remove_git_env()
+                .check(true)
+                .output()
+                .await?;
         }
 
+        reporter.on_install_complete(progress);
+
         Ok(InstalledHook::Installed {
             hook,
             info: Arc::new(info),
@@ -113,8 +132,8 @@ impl LanguageImpl for Golang {
         let go_bin = bin_dir(env_dir);
         let new_path = prepend_paths(&[&go_bin, go_root_bin]).context("Failed to join PATH")?;
 
-        let entry = hook.entry.parsed()?;
-        let run = async move |batch: Vec<String>| {
+        let entry = hook.entry.resolve(Some(&new_path))?;
+        let run = async move |batch: &[&String]| {
             let mut output = Cmd::new(&entry[0], "go hook")
                 .args(&entry[1..])
                 .env("PATH", &new_path)
@@ -125,7 +144,7 @@ impl LanguageImpl for Golang {
                 .args(&hook.args)
                 .args(batch)
                 .check(false)
-                .output()
+                .pty_output()
                 .await?;
 
             output.stdout.extend(output.stderr);
