@@ -1,15 +1,21 @@
-use std::path::Path;
+use std::ffi::OsStr;
+use std::path::{Path, PathBuf};
 use std::sync::Arc;
 
 use anyhow::{Context, Result};
 use futures::TryStreamExt;
+use http::header::USER_AGENT;
 use tokio_util::compat::FuturesAsyncReadCompatExt;
-use tracing::trace;
+use tracing::{debug, trace};
 
 use crate::archive::ArchiveExtension;
+use crate::cli::reporter::HookInstallReporter;
 use crate::config::Language;
+use crate::fs::CWD;
 use crate::hook::{Hook, InstalledHook};
-use crate::store::Store;
+use crate::identify::parse_shebang;
+use crate::store::{STORE, Store};
+use crate::version::version;
 use crate::{archive, builtin};
 
 mod docker;
@@ -35,7 +41,12 @@ static PYGREP: pygrep::Pygrep = pygrep::Pygrep;
 static UNIMPLEMENTED: Unimplemented = Unimplemented;
 
 trait LanguageImpl {
-    async fn install(&self, hook: Arc<Hook>, store: &Store) -> Result<InstalledHook>;
+    async fn install(
+        &self,
+        hook: Arc<Hook>,
+        store: &Store,
+        reporter: &HookInstallReporter,
+    ) -> Result<InstalledHook>;
     async fn check_health(&self) -> Result<()>;
     async fn run(
         &self,
@@ -52,7 +63,12 @@ struct UnimplementedError(String);
 struct Unimplemented;
 
 impl LanguageImpl for Unimplemented {
-    async fn install(&self, hook: Arc<Hook>, _store: &Store) -> Result<InstalledHook> {
+    async fn install(
+        &self,
+        hook: Arc<Hook>,
+        _store: &Store,
+        _reporter: &HookInstallReporter,
+    ) -> Result<InstalledHook> {
         Ok(InstalledHook::NoNeedInstall(hook))
     }
 
@@ -143,18 +159,23 @@ impl Language {
         )
     }
 
-    pub async fn install(&self, hook: Arc<Hook>, store: &Store) -> Result<InstalledHook> {
+    pub async fn install(
+        &self,
+        hook: Arc<Hook>,
+        store: &Store,
+        reporter: &HookInstallReporter,
+    ) -> Result<InstalledHook> {
         match self {
-            Self::Golang => GOLANG.install(hook, store).await,
-            Self::Python => PYTHON.install(hook, store).await,
-            Self::Node => NODE.install(hook, store).await,
-            Self::System => SYSTEM.install(hook, store).await,
-            Self::Fail => FAIL.install(hook, store).await,
-            Self::Docker => DOCKER.install(hook, store).await,
-            Self::DockerImage => DOCKER_IMAGE.install(hook, store).await,
-            Self::Script => SCRIPT.install(hook, store).await,
-            Self::Pygrep => PYGREP.install(hook, store).await,
-            _ => UNIMPLEMENTED.install(hook, store).await,
+            Self::Golang => GOLANG.install(hook, store, reporter).await,
+            Self::Python => PYTHON.install(hook, store, reporter).await,
+            Self::Node => NODE.install(hook, store, reporter).await,
+            Self::System => SYSTEM.install(hook, store, reporter).await,
+            Self::Fail => FAIL.install(hook, store, reporter).await,
+            Self::Docker => DOCKER.install(hook, store, reporter).await,
+            Self::DockerImage => DOCKER_IMAGE.install(hook, store, reporter).await,
+            Self::Script => SCRIPT.install(hook, store, reporter).await,
+            Self::Pygrep => PYGREP.install(hook, store, reporter).await,
+            _ => UNIMPLEMENTED.install(hook, store, reporter).await,
         }
     }
 
@@ -199,86 +220,54 @@ impl Language {
     }
 }
 
-/// Create a symlink or copy the file on Windows.
-/// Tries symlink first, falls back to copy if symlink fails.
-async fn create_symlink_or_copy(source: &Path, target: &Path) -> Result<()> {
-    if target.exists() {
-        fs_err::tokio::remove_file(target).await?;
+/// Try to extract metadata from the given hook entry if possible.
+///
+/// Currently, only PEP 723 inline metadata for `python` hooks is supported.
+/// First part of `entry` must be a file path to the Python script.
+/// Effectively, we are implementing a new `python-script` language which works like `script`.
+/// But we don't want to introduce a new language just for this for now.
+pub(crate) async fn extract_metadata_from_entry(hook: &mut Hook) -> Result<()> {
+    // Only support `python` hooks for now.
+    if hook.language == Language::Python {
+        return python::extract_pep723_metadata(hook).await;
     }
 
-    #[cfg(not(windows))]
-    {
-        // Try symlink on Unix systems
-        match fs_err::tokio::symlink(source, target).await {
-            Ok(()) => {
-                trace!(
-                    "Created symlink from {} to {}",
-                    source.display(),
-                    target.display()
-                );
-                return Ok(());
-            }
-            Err(e) => {
-                trace!(
-                    "Failed to create symlink from {} to {}: {}",
-                    source.display(),
-                    target.display(),
-                    e
-                );
-            }
-        }
-    }
-
-    #[cfg(windows)]
-    {
-        // Try Windows symlink API (requires admin privileges)
-        use std::os::windows::fs::symlink_file;
-        match symlink_file(source, target) {
-            Ok(()) => {
-                trace!(
-                    "Created Windows symlink from {} to {}",
-                    source.display(),
-                    target.display()
-                );
-                return Ok(());
-            }
-            Err(e) => {
-                trace!(
-                    "Failed to create Windows symlink from {} to {}: {}",
-                    source.display(),
-                    target.display(),
-                    e
-                );
-            }
-        }
-    }
-
-    // Fallback to copy
-    trace!(
-        "Falling back to copy from {} to {}",
-        source.display(),
-        target.display()
-    );
-    fs_err::tokio::copy(source, target).await.with_context(|| {
-        format!(
-            "Failed to copy file from {} to {}",
-            source.display(),
-            target.display(),
-        )
-    })?;
-
     Ok(())
 }
 
+pub(crate) fn resolve_command(mut cmds: Vec<String>, env_path: Option<&OsStr>) -> Vec<String> {
+    let cmd = &cmds[0];
+    let exe_path = match which::which_in(cmd, env_path, &*CWD) {
+        Ok(p) => p,
+        Err(_) => PathBuf::from(cmd),
+    };
+    trace!("Resolved command: {}", exe_path.display());
+
+    if let Ok(mut interpreter) = parse_shebang(&exe_path) {
+        trace!("Found shebang: {:?}", interpreter);
+        // Resolve the interpreter path, convert "python3" to "python3.exe" on Windows
+        if let Ok(p) = which::which_in(&interpreter[0], env_path, &*CWD) {
+            interpreter[0] = p.to_string_lossy().to_string();
+            trace!("Resolved interpreter: {}", &interpreter[0]);
+        }
+        interpreter.push(exe_path.to_string_lossy().to_string());
+        interpreter.extend_from_slice(&cmds[1..]);
+        interpreter
+    } else {
+        cmds[0] = exe_path.to_string_lossy().to_string();
+        cmds
+    }
+}
+
 async fn download_and_extract(
     client: &reqwest::Client,
     url: &str,
-    target: &Path,
     filename: &str,
-    scratch: &Path,
+    callback: impl AsyncFn(&Path) -> Result<()>,
 ) -> Result<()> {
     let response = client
         .get(url)
+        .header(USER_AGENT, format!("prek/{}", version()))
         .send()
         .await
         .with_context(|| format!("Failed to download file from {url}"))?;
@@ -296,26 +285,24 @@ async fn download_and_extract(
         .into_async_read()
         .compat();
 
-    let temp_dir = tempfile::tempdir_in(scratch)?;
-    trace!(url = %url, temp_dir = ?temp_dir.path(), "Downloading");
+    let scratch = STORE.as_ref()?.scratch_path();
+    fs_err::tokio::create_dir_all(&scratch).await?;
+
+    let temp_dir = tempfile::tempdir_in(&scratch)?;
+    debug!(url = %url, temp_dir = ?temp_dir.path(), "Downloading");
 
     let ext = ArchiveExtension::from_path(filename)?;
     archive::unpack(tarball, ext, temp_dir.path()).await?;
 
     let extracted = match archive::strip_component(temp_dir.path()) {
         Ok(top_level) => top_level,
-        Err(archive::Error::NonSingularArchive(_)) => temp_dir.keep(),
+        Err(archive::Error::NonSingularArchive(_)) => temp_dir.path().to_path_buf(),
         Err(err) => return Err(err.into()),
     };
 
-    if target.is_dir() {
-        trace!(target = %target.display(), "Removing existing target");
-        fs_err::tokio::remove_dir_all(&target).await?;
-    }
+    callback(&extracted).await?;
 
-    trace!(temp_dir = ?extracted, target = %target.display(), "Moving to target");
-    // TODO: retry on Windows
-    fs_err::tokio::rename(extracted, target).await?;
+    fs_err::tokio::remove_dir_all(temp_dir.path()).await?;
 
     Ok(())
 }
