@@ -3,10 +3,11 @@ use std::path::{Path, PathBuf};
 use std::sync::Arc;
 
 use anyhow::{Context, Result};
-use tracing::debug;
+use tracing::{debug, trace};
 
 use constants::env_vars::EnvVars;
 
+use crate::cli::reporter::HookInstallReporter;
 use crate::hook::InstalledHook;
 use crate::hook::{Hook, InstallInfo};
 use crate::languages::LanguageImpl;
@@ -45,9 +46,16 @@ fn to_uv_python_request(request: &LanguageRequest) -> Option<String> {
 }
 
 impl LanguageImpl for Python {
-    async fn install(&self, hook: Arc<Hook>, store: &Store) -> Result<InstalledHook> {
+    async fn install(
+        &self,
+        hook: Arc<Hook>,
+        store: &Store,
+        reporter: &HookInstallReporter,
+    ) -> Result<InstalledHook> {
+        let progress = reporter.on_install_start(&hook);
+
         let uv_dir = store.tools_path(ToolBucket::Uv);
-        let uv = Uv::install(&uv_dir).await?;
+        let uv = Uv::install(&uv_dir).await.context("Failed to install uv")?;
 
         let mut info = InstallInfo::new(
             hook.language,
@@ -66,6 +74,10 @@ impl LanguageImpl for Python {
 
         // Install dependencies
         if let Some(repo_path) = hook.repo_path() {
+            trace!(
+                "Installing dependencies from repo path: {}",
+                repo_path.display()
+            );
             uv.cmd("uv pip install", store)
                 .arg("pip")
                 .arg("install")
@@ -77,6 +89,7 @@ impl LanguageImpl for Python {
                 .output()
                 .await?;
         } else if !hook.additional_dependencies.is_empty() {
+            trace!("Installing additional dependencies for local hook");
             uv.cmd("uv pip install", store)
                 .arg("pip")
                 .arg("install")
@@ -86,7 +99,7 @@ impl LanguageImpl for Python {
                 .output()
                 .await?;
         } else {
-            debug!("No dependencies to install");
+            debug!("No additional dependencies to install");
         }
 
         let python = python_exec(&info.env_path);
@@ -116,6 +129,8 @@ impl LanguageImpl for Python {
         info.with_language_version(version)
             .with_toolchain(python_exec);
 
+        reporter.on_install_complete(progress);
+
         Ok(InstalledHook::Installed {
             hook,
             info: Arc::new(info),
@@ -134,10 +149,9 @@ impl LanguageImpl for Python {
     ) -> Result<(i32, Vec<u8>)> {
         let env_dir = hook.env_path().expect("Python must have env path");
         let new_path = prepend_paths(&[&bin_dir(env_dir)]).context("Failed to join PATH")?;
-        let entry = hook.entry.parsed()?;
+        let entry = hook.entry.resolve(Some(&new_path))?;
 
-        let run = async move |batch: Vec<String>| {
-            // TODO: combine stdout and stderr
+        let run = async move |batch: &[&String]| {
             let mut output = Cmd::new(&entry[0], "python hook")
                 .args(&entry[1..])
                 .env("VIRTUAL_ENV", env_dir)
@@ -146,7 +160,7 @@ impl LanguageImpl for Python {
                 .args(&hook.args)
                 .args(batch)
                 .check(false)
-                .output()
+                .pty_output()
                 .await?;
 
             output.stdout.extend(output.stderr);
