@@ -2,20 +2,28 @@ use std::fmt::Write;
 use std::path::Path;
 use std::process::ExitCode;
 use std::str::FromStr;
+use std::sync::Mutex;
 
-use anstream::{ColorChoice, eprintln};
+use anstream::{ColorChoice, StripStream, eprintln};
 use anyhow::{Context, Result};
 use clap::{CommandFactory, Parser};
 use clap_complete::CompleteEnv;
 use owo_colors::OwoColorize;
+use tracing::level_filters::LevelFilter;
 use tracing::{debug, error};
-use tracing_subscriber::EnvFilter;
 use tracing_subscriber::filter::Directive;
+use tracing_subscriber::layer::SubscriberExt;
+use tracing_subscriber::util::SubscriberInitExt;
+use tracing_subscriber::{EnvFilter, Layer};
 
 use crate::cleanup::cleanup;
-use crate::cli::{Cli, Command, ExitStatus, SelfCommand, SelfNamespace, SelfUpdateArgs};
-use crate::git::get_root;
+use crate::cli::{Cli, Command, ExitStatus};
+#[cfg(feature = "self-update")]
+use crate::cli::{SelfCommand, SelfNamespace, SelfUpdateArgs};
+use crate::git::GIT_ROOT;
 use crate::printer::Printer;
+use crate::run::USE_COLOR;
+use crate::store::STORE;
 
 mod archive;
 mod builtin;
@@ -31,6 +39,8 @@ mod printer;
 mod process;
 #[cfg(all(unix, feature = "profiler"))]
 mod profiler;
+#[cfg(not(windows))]
+mod pty;
 mod run;
 mod store;
 mod version;
@@ -54,36 +64,63 @@ pub(crate) enum Level {
 
 fn setup_logging(level: Level) -> Result<()> {
     let directive = match level {
-        Level::Default | Level::Verbose => tracing::level_filters::LevelFilter::OFF.into(),
+        Level::Default | Level::Verbose => LevelFilter::OFF.into(),
         Level::Debug => Directive::from_str("prek=debug")?,
         Level::Trace => Directive::from_str("prek=trace")?,
         Level::TraceAll => Directive::from_str("trace")?,
     };
 
-    let filter = EnvFilter::builder()
+    let stderr_filter = EnvFilter::builder()
         .with_default_directive(directive)
         .from_env()
         .context("Invalid RUST_LOG directive")?;
-
-    let ansi = match anstream::Stderr::choice(&std::io::stderr()) {
-        ColorChoice::Always | ColorChoice::AlwaysAnsi => true,
-        ColorChoice::Never => false,
-        // We just asked anstream for a choice, that can't be auto
-        ColorChoice::Auto => unreachable!(),
-    };
-
-    let format = tracing_subscriber::fmt::format()
+    let stderr_format = tracing_subscriber::fmt::format()
         .with_target(false)
         .without_time()
-        .with_ansi(ansi);
-    tracing_subscriber::fmt::fmt()
-        .with_env_filter(filter)
-        .event_format(format)
+        .with_ansi(*USE_COLOR);
+    let stderr_layer = tracing_subscriber::fmt::layer()
+        .event_format(stderr_format)
         .with_writer(anstream::stderr)
+        .with_filter(stderr_filter);
+
+    let log_file_path = STORE.as_ref()?.log_file();
+    let log_file = fs_err::OpenOptions::new()
+        .create(true)
+        .write(true)
+        .open(log_file_path)
+        .context("Failed to open log file")?;
+    let log_file = Mutex::new(StripStream::new(log_file.into_file()));
+
+    let file_format = tracing_subscriber::fmt::format()
+        .with_target(false)
+        .with_ansi(false);
+    let file_layer = tracing_subscriber::fmt::layer()
+        .event_format(file_format)
+        .with_writer(log_file)
+        .with_filter(EnvFilter::new("prek=trace"));
+
+    tracing_subscriber::registry()
+        .with(stderr_layer)
+        .with(file_layer)
         .init();
+
     Ok(())
 }
 
+fn should_change_cwd(cli: &Cli) -> bool {
+    cli.command.as_ref().is_some_and(|cmd| {
+        !matches!(
+            cmd,
+            Command::Clean
+                | Command::GC
+                | Command::InitTemplateDir(_)
+                | Command::SampleConfig(_)
+                | Command::ValidateConfig(_)
+                | Command::ValidateManifest(_)
+        )
+    })
+}
+
 /// Adjusts relative paths in the CLI arguments to be relative to the new working directory.
 fn adjust_relative_paths(cli: &mut Cli, new_cwd: &Path) -> Result<()> {
     if let Some(path) = &mut cli.globals.config {
@@ -92,6 +129,7 @@ fn adjust_relative_paths(cli: &mut Cli, new_cwd: &Path) -> Result<()> {
         }
     }
 
+    // Adjust path arguments for `run` and `try-repo` commands.
     if let Some(Command::Run(ref mut args) | Command::TryRepo(ref mut args)) = cli.command {
         args.files = args
             .files
@@ -156,17 +194,18 @@ async fn run(mut cli: Cli) -> Result<ExitStatus> {
 
     debug!("prek: {}", version::version());
 
-    match get_root().await {
-        Ok(root) => {
-            debug!("Git root: {}", root.display());
+    // Adjust relative paths before changing the working directory.
+    if should_change_cwd(&cli) {
+        match GIT_ROOT.as_ref() {
+            Ok(root) => {
+                debug!("Git root: {}", root.display());
+                adjust_relative_paths(&mut cli, root)?;
 
-            // Adjust relative paths before changing the working directory.
-            adjust_relative_paths(&mut cli, &root)?;
-
-            std::env::set_current_dir(&root)?;
-        }
-        Err(err) => {
-            error!("Failed to find git root: {}", err);
+                std::env::set_current_dir(root)?;
+            }
+            Err(err) => {
+                error!("Failed to find git root: {}", err);
+            }
         }
     }
 
@@ -235,6 +274,7 @@ async fn run(mut cli: Cli) -> Result<ExitStatus> {
                 args.hook_ids,
                 args.hook_stage,
                 args.language,
+                args.output_format,
                 printer,
             )
             .await
@@ -264,6 +304,18 @@ async fn run(mut cli: Cli) -> Result<ExitStatus> {
             Ok(cli::validate_manifest(args.manifests))
         }
         Command::SampleConfig(args) => cli::sample_config(args.file, printer),
+        Command::AutoUpdate(args) => {
+            cli::auto_update(
+                cli.globals.config,
+                args.repo,
+                args.bleeding_edge,
+                args.freeze,
+                args.jobs,
+                printer,
+            )
+            .await
+        }
+        #[cfg(feature = "self-update")]
         Command::Self_(SelfNamespace {
             command:
                 SelfCommand::Update(SelfUpdateArgs {
@@ -271,6 +323,14 @@ async fn run(mut cli: Cli) -> Result<ExitStatus> {
                     token,
                 }),
         }) => cli::self_update(target_version, token, printer).await,
+        #[cfg(not(feature = "self-update"))]
+        Command::Self_(_) => {
+            anyhow::bail!(
+                "prek was installed through an external package manager, and self-update \
+                is not available. Please use your package manager to update prek."
+            );
+        }
+
         Command::GenerateShellCompletion(args) => {
             show_settings!(args);
 
