@@ -14,6 +14,7 @@ use crate::config::RemoteRepo;
 use crate::fs::LockedFile;
 use crate::git::clone_repo;
 use crate::hook::InstallInfo;
+use crate::workspace::HookInitReporter;
 
 #[derive(Debug, Error)]
 pub enum Error {
@@ -22,27 +23,26 @@ pub enum Error {
     #[error(transparent)]
     Io(#[from] std::io::Error),
     #[error(transparent)]
-    Fmt(#[from] std::fmt::Error),
-    #[error(transparent)]
-    Repo(#[from] crate::hook::Error),
-    #[error(transparent)]
     Git(#[from] crate::git::Error),
     #[error(transparent)]
     Serde(#[from] serde_json::Error),
 }
 
-static STORE_HOME: LazyLock<Option<PathBuf>> = LazyLock::new(|| {
-    if let Some(path) = EnvVars::var_os(EnvVars::PREK_HOME) {
-        debug!(
-            path = %path.to_string_lossy(),
-            "Loading store from PREK_HOME env var",
-        );
+pub(crate) static STORE: LazyLock<Result<Store, Error>> = LazyLock::new(|| {
+    let path = if let Some(path) = EnvVars::var_os(EnvVars::PREK_HOME) {
         Some(path.into())
     } else {
         etcetera::choose_base_strategy()
             .map(|path| path.cache_dir().join("prek"))
             .ok()
-    }
+    };
+
+    let Some(path) = path else {
+        return Err(Error::HomeNotFound);
+    };
+    let store = Store::from_path(path).init()?;
+
+    Ok(store)
 });
 
 /// A store for managing repos.
@@ -52,12 +52,6 @@ pub struct Store {
 }
 
 impl Store {
-    pub(crate) fn from_settings() -> Result<Self, Error> {
-        Ok(Self::from_path(
-            STORE_HOME.as_ref().ok_or(Error::HomeNotFound)?,
-        ))
-    }
-
     pub(crate) fn from_path(path: impl Into<PathBuf>) -> Self {
         Self { path: path.into() }
     }
@@ -82,23 +76,32 @@ impl Store {
     }
 
     /// Clone a remote repo into the store.
-    pub(crate) async fn clone_repo(&self, repo: &RemoteRepo) -> Result<PathBuf, Error> {
+    pub(crate) async fn clone_repo(
+        &self,
+        repo: &RemoteRepo,
+        reporter: Option<&dyn HookInitReporter>,
+    ) -> Result<PathBuf, Error> {
         // Check if the repo is already cloned.
         let target = self.repo_path(repo);
         if target.join(".prek-repo.json").try_exists()? {
             return Ok(target);
         }
 
+        let progress =
+            reporter.map(|reporter| (reporter, reporter.on_clone_start(&format!("{repo}"))));
+
         fs_err::tokio::create_dir_all(self.repos_dir()).await?;
 
         // Clone and checkout the repo.
-        let temp = tempfile::tempdir_in(self.repos_dir())?;
+        fs_err::tokio::create_dir_all(self.scratch_path()).await?;
+        let temp = tempfile::tempdir_in(self.scratch_path())?;
+
         debug!(
             target = %temp.path().display(),
-            %repo,
+            ?repo,
             "Cloning repo",
         );
-        clone_repo(repo.repo.as_str(), &repo.rev, temp.path()).await?;
+        clone_repo(&repo.repo, &repo.rev, temp.path()).await?;
 
         // TODO: add windows retry
         fs_err::tokio::remove_dir_all(&target).await.ok();
@@ -107,6 +110,10 @@ impl Store {
         let content = serde_json::to_string_pretty(&repo)?;
         fs_err::tokio::write(target.join(".prek-repo.json"), content).await?;
 
+        if let Some((reporter, progress)) = progress {
+            reporter.on_clone_complete(progress);
+        }
+
         Ok(target)
     }
 
@@ -161,6 +168,15 @@ impl Store {
     pub(crate) fn cache_path(&self, tool: CacheBucket) -> PathBuf {
         self.path.join("cache").join(tool.as_str())
     }
+
+    /// Scratch path for temporary files.
+    pub(crate) fn scratch_path(&self) -> PathBuf {
+        self.path.join("scratch")
+    }
+
+    pub(crate) fn log_file(&self) -> PathBuf {
+        self.path.join("prek.log")
+    }
 }
 
 #[derive(Copy, Clone)]
