@@ -184,6 +184,12 @@ impl TestContext {
         command
     }
 
+    pub fn auto_update(&self) -> Command {
+        let mut cmd = self.command();
+        cmd.arg("auto-update");
+        cmd
+    }
+
     /// Standard snapshot filters _plus_ those for this test context.
     pub fn filters(&self) -> Vec<(&str, &str)> {
         // Put test context snapshots before the default filters
@@ -233,6 +239,17 @@ impl TestContext {
             .success();
     }
 
+    pub fn disable_auto_crlf(&self) {
+        // Disable autocrlf
+        Command::new("git")
+            .arg("config")
+            .arg("core.autocrlf")
+            .arg("false")
+            .current_dir(&self.temp_dir)
+            .assert()
+            .success();
+    }
+
     /// Run `git add`.
     pub fn git_add(&self, path: impl AsRef<OsStr>) {
         Command::new("git")
