@@ -118,12 +118,13 @@ fn language_version() -> anyhow::Result<()> {
     Ok(())
 }
 
-/// Test that `additional_dependencies` are installed correctly.
+/// Test a remote go hook.
 #[test]
-fn additional_dependencies() {
+fn remote_hook() {
     let context = TestContext::new();
     context.init_project();
 
+    // Test that `additional_dependencies` are installed correctly.
     context.write_pre_commit_config(indoc::indoc! {r#"
         repos:
           - repo: local
@@ -138,7 +139,6 @@ fn additional_dependencies() {
                 language_version: '1.23.11' # will auto download
                 pass_filenames: false
     "#});
-
     context.git_add(".");
 
     cmd_snapshot!(context.filters(), context.run(), @r#"
@@ -162,19 +162,12 @@ fn additional_dependencies() {
 
     ----- stderr -----
     "#);
-}
-
-/// Test a remote go hook.
-#[test]
-fn remote_hook() {
-    let context = TestContext::new();
-    context.init_project();
 
     // Run hooks with newly downloaded go.
     context.write_pre_commit_config(indoc::indoc! {r"
         repos:
           - repo: https://github.com/prek-test-repos/golang-hooks
-            rev: main
+            rev: v1.0
             hooks:
               - id: echo
                 verbose: true
@@ -182,7 +175,7 @@ fn remote_hook() {
         "});
     context.git_add(".");
 
-    cmd_snapshot!(context.filters(), context.run(), @r#"
+    cmd_snapshot!(context.filters(), context.run(), @r"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -192,13 +185,13 @@ fn remote_hook() {
       .pre-commit-config.yaml
 
     ----- stderr -----
-    "#);
+    ");
 
     // Run hooks with system found go.
     context.write_pre_commit_config(indoc::indoc! {r"
         repos:
           - repo: https://github.com/prek-test-repos/golang-hooks
-            rev: main
+            rev: v1.0
             hooks:
               - id: echo
                 verbose: true
@@ -206,7 +199,7 @@ fn remote_hook() {
         "});
     context.git_add(".");
 
-    cmd_snapshot!(context.filters(), context.run(), @r#"
+    cmd_snapshot!(context.filters(), context.run(), @r"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -216,5 +209,5 @@ fn remote_hook() {
       .pre-commit-config.yaml
 
     ----- stderr -----
-    "#);
+    ");
 }
