@@ -1,10 +1,10 @@
 use assert_fs::assert::PathAssert;
-use assert_fs::fixture::PathChild;
+use assert_fs::fixture::{FileWriteStr, PathChild};
 
 use crate::common::{TestContext, cmd_snapshot};
 
 /// Test `language_version` parsing.
-/// Python 3.12.11 and 3.13.5 are installed in the CI environment, when running tests uv can find them.
+/// Python 3.12.11 is installed in the CI environment, when running tests uv can find them.
 /// Other versions may need to be downloaded while running the tests.
 #[test]
 fn language_version() -> anyhow::Result<()> {
@@ -49,12 +49,6 @@ fn language_version() -> anyhow::Result<()> {
                 entry: python -c 'import sys; print(sys.version_info[:3])'
                 language_version: python3.12
                 always_run: true
-              - id: greater-than-python3.13
-                name: greater-than-python3.13
-                language: python
-                entry: python -c 'import sys; print(sys.version_info[:3])'
-                language_version: '>=3.13'
-                always_run: true
               - id: python3.12
                 name: python3.12
                 language: python
@@ -98,10 +92,6 @@ fn language_version() -> anyhow::Result<()> {
     - hook id: python3.12
     - duration: [TIME]
       (3, 12, 11)
-    greater-than-python3.13..................................................Passed
-    - hook id: greater-than-python3.13
-    - duration: [TIME]
-      (3, 13, 5)
     python3.12...............................................................Passed
     - hook id: python3.12
     - duration: [TIME]
@@ -241,3 +231,100 @@ fn additional_dependencies() {
     ----- stderr -----
     "#);
 }
+
+/// Ensure that stderr from hooks is captured and shown to the user.
+#[test]
+fn hook_stderr() -> anyhow::Result<()> {
+    let context = TestContext::new();
+    context.init_project();
+
+    context.write_pre_commit_config(indoc::indoc! {r"
+        repos:
+          - repo: local
+            hooks:
+              - id: local
+                name: local
+                language: python
+                entry: python ./hook.py
+    "});
+
+    context
+        .work_dir()
+        .child("hook.py")
+        .write_str("import sys; print('How are you', file=sys.stderr); sys.exit(1)")?;
+
+    context.git_add(".");
+
+    cmd_snapshot!(context.filters(), context.run(), @r#"
+    success: false
+    exit_code: 1
+    ----- stdout -----
+    local....................................................................Failed
+    - hook id: local
+    - exit code: 1
+      How are you
+
+    ----- stderr -----
+    "#);
+
+    Ok(())
+}
+
+/// Test that pep723 script for local hook is installed correctly.
+/// Only if no additional dependencies are specified.
+#[test]
+fn pep723_script() -> anyhow::Result<()> {
+    let context = TestContext::new();
+    context.init_project();
+    context.write_pre_commit_config(indoc::indoc! {r#"
+        repos:
+          - repo: local
+            hooks:
+              - id: other-hook
+                name: other-hook
+                language: python
+                entry: python -c 'print("hello from other-hook")'
+                verbose: true
+                pass_filenames: false
+              - id: local
+                name: local
+                language: python
+                entry: ./script.py hello world
+                verbose: true
+                pass_filenames: false
+    "#});
+    // On Windows, uv venv does not create `python3.exe`, `python3.12.exe` symlink,
+    // be sure to use `python` as the interpreter name.
+    context
+        .work_dir()
+        .child("script.py")
+        .write_str(indoc::indoc! {r#"
+        #!/usr/bin/env python
+        # /// script
+        # requires-python = ">=3.10"
+        # dependencies = [ "pyecho-cli" ]
+        # ///
+        from pyecho import main
+        main()
+    "#})?;
+
+    context.git_add(".");
+
+    cmd_snapshot!(context.filters(), context.run(), @r#"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    other-hook...............................................................Passed
+    - hook id: other-hook
+    - duration: [TIME]
+      hello from other-hook
+    local....................................................................Passed
+    - hook id: local
+    - duration: [TIME]
+      hello world
+
+    ----- stderr -----
+    "#);
+
+    Ok(())
+}
