@@ -265,7 +265,7 @@ fn multiple_hook_ids() {
     ----- stdout -----
 
     ----- stderr -----
-    No hook found with id `nonexistent-hook` in stage `pre-commit`
+    error: No hook found with id `nonexistent-hook` in stage `pre-commit`
     "#);
 
     // Multiple hook_ids match nothing
@@ -275,7 +275,7 @@ fn multiple_hook_ids() {
     ----- stdout -----
 
     ----- stderr -----
-    No hooks found with ids `nonexistent-hook`, `nonexistent-hook-2` in stage `pre-commit`
+    error: No hooks found with ids `nonexistent-hook`, `nonexistent-hook-2` in stage `pre-commit`
     "#);
 
     // Hook-id matches one hook
@@ -297,10 +297,11 @@ fn multiple_hook_ids() {
     Second Hook..............................................................Passed
 
     ----- stderr -----
+    warning: Ignored non-existent hook ID: `nonexistent`
     "#);
 
     // Multiple valid hook-ids
-    cmd_snapshot!(context.filters(), context.run().arg("hook1").arg("hook2"), @r#"
+    cmd_snapshot!(context.filters(), context.run().arg("hook1").arg("hook2").arg("nonexistent-hook"), @r#"
     success: true
     exit_code: 0
     ----- stdout -----
@@ -308,6 +309,7 @@ fn multiple_hook_ids() {
     Second Hook..............................................................Passed
 
     ----- stderr -----
+    warning: Ignored non-existent hook ID: `nonexistent-hook`
     "#);
 
     // Multiple hook-ids with some duplicates and aliases
@@ -351,8 +353,7 @@ fn config_not_staged() -> Result<()> {
     ----- stdout -----
 
     ----- stderr -----
-    Your pre-commit configuration file is not staged.
-    Run `git add .pre-commit-config.yaml` to fix this.
+    error: prek configuration file is not staged, run `git add .pre-commit-config.yaml` to stage it
     "#);
 
     Ok(())
@@ -1037,7 +1038,7 @@ fn merge_conflicts() -> Result<()> {
     ----- stdout -----
 
     ----- stderr -----
-    You have unmerged paths. Resolve them before running prek.
+    error: You have unmerged paths. Resolve them before running prek
     "#);
 
     // Fix the conflict and run again.
@@ -1088,37 +1089,6 @@ fn local_python_hook() {
     "#);
 }
 
-/// Supports reading `pre-commit-config.yml` as well.
-#[test]
-fn alternate_config_file() {
-    let context = TestContext::new();
-    context.init_project();
-
-    context.write_pre_commit_config(indoc::indoc! {r#"
-        repos:
-          - repo: local
-            hooks:
-              - id: local-python-hook
-                name: local-python-hook
-                language: python
-                entry: python3 -c 'import sys; print("Hello, world!")'
-    "#});
-
-    context.git_add(".");
-
-    cmd_snapshot!(context.filters(), context.run().arg("-v"), @r#"
-    success: true
-    exit_code: 0
-    ----- stdout -----
-    local-python-hook........................................................Passed
-    - hook id: local-python-hook
-    - duration: [TIME]
-      Hello, world!
-
-    ----- stderr -----
-    "#);
-}
-
 /// Invalid `entry`
 #[test]
 fn invalid_entry() {
@@ -1443,3 +1413,298 @@ fn run_directory() -> Result<()> {
 
     Ok(())
 }
+
+/// Test `minimum_prek_version` option.
+#[test]
+fn minimum_prek_version() {
+    let context = TestContext::new();
+    context.init_project();
+    context.write_pre_commit_config(indoc::indoc! {r"
+        minimum_prek_version: 10.0.0
+        repos:
+          - repo: local
+            hooks:
+              - id: directory
+                name: directory
+                language: system
+                entry: echo
+                verbose: true
+    "});
+    context.git_add(".");
+
+    let filters = context
+        .filters()
+        .into_iter()
+        .chain([(
+            r"current version `\d+\.\d+\.\d+`",
+            "current version `[CURRENT_VERSION]`",
+        )])
+        .collect::<Vec<_>>();
+
+    cmd_snapshot!(filters, context.run(), @r#"
+    success: false
+    exit_code: 2
+    ----- stdout -----
+
+    ----- stderr -----
+    error: Failed to parse `.pre-commit-config.yaml`
+      caused by: Required minimum prek version `10.0.0` is greater than current version `[CURRENT_VERSION]`. Please consider updating prek.
+    "#);
+}
+
+/// Run hooks that would echo color.
+#[test]
+#[cfg(not(windows))]
+fn color() -> Result<()> {
+    let context = TestContext::new();
+    context.init_project();
+    context.write_pre_commit_config(indoc::indoc! {r"
+      repos:
+        - repo: local
+          hooks:
+            - id: color
+              name: color
+              language: python
+              entry: python ./color.py
+              verbose: true
+              pass_filenames: false
+  "});
+
+    let script = indoc::indoc! {r"
+      import sys
+      if sys.stdout.isatty():
+          print('\033[1;32mHello, world!\033[0m')
+      else:
+          print('Hello, world!')
+  "};
+    context.work_dir().child("color.py").write_str(script)?;
+
+    context.git_add(".");
+
+    // Run default. In integration tests, we don't have a TTY.
+    // So this prints without color.
+    cmd_snapshot!(context.filters(), context.run(), @r#"
+  success: true
+  exit_code: 0
+  ----- stdout -----
+  color....................................................................Passed
+  - hook id: color
+  - duration: [TIME]
+    Hello, world!
+
+  ----- stderr -----
+  "#);
+
+    // Force color output
+    cmd_snapshot!(context.filters(), context.run().arg("--color=always"), @r#"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    color....................................................................[42mPassed[49m
+    [2m- hook id: color[0m
+    [2m- duration: [TIME][0m
+    [2m  [1;32mHello, world![0m[0m
+
+    ----- stderr -----
+    "#);
+
+    Ok(())
+}
+
+/// Test running hook whose `entry` is script with shebang on Windows.
+#[test]
+fn shebang_script() -> Result<()> {
+    let context = TestContext::new();
+    context.init_project();
+
+    // Create a script with shebang.
+    let script = indoc::indoc! {r"
+        #!/usr/bin/env python
+        import sys
+        print('Hello, world!')
+        sys.exit(0)
+    "};
+    context.work_dir().child("script.py").write_str(script)?;
+
+    context.write_pre_commit_config(indoc::indoc! {r"
+      repos:
+        - repo: local
+          hooks:
+            - id: shebang-script
+              name: shebang-script
+              language: python
+              entry: script.py
+              verbose: true
+              pass_filenames: false
+              always_run: true
+    "});
+    context.git_add(".");
+
+    cmd_snapshot!(context.filters(), context.run(), @r#"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    shebang-script...........................................................Passed
+    - hook id: shebang-script
+    - duration: [TIME]
+      Hello, world!
+
+    ----- stderr -----
+    "#);
+
+    Ok(())
+}
+
+/// Test `git commit -a` works without `.git/index.lock exists` error.
+#[test]
+fn git_commit_a() -> Result<()> {
+    let context = TestContext::new();
+    context.init_project();
+    context.configure_git_author();
+    context.disable_auto_crlf();
+
+    context.write_pre_commit_config(indoc::indoc! {r"
+        repos:
+          - repo: local
+            hooks:
+              - id: echo
+                name: echo
+                language: system
+                entry: echo
+                verbose: true
+    "});
+
+    // Create a file and commit it.
+    let cwd = context.work_dir();
+    let file = cwd.child("file.txt");
+    file.write_str("Hello, world!\n")?;
+
+    cmd_snapshot!(context.filters(), context.install(), @r#"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    prek installed at `.git/hooks/pre-commit`
+
+    ----- stderr -----
+    "#);
+
+    context.git_add(".");
+    context.git_commit("Initial commit");
+
+    // Edit the file
+    file.write_str("Hello, world again!\n")?;
+
+    let mut commit = Command::new("git");
+    commit
+        .arg("commit")
+        .arg("-a")
+        .arg("-m")
+        .arg("Update file")
+        .current_dir(cwd);
+
+    let filters = context
+        .filters()
+        .into_iter()
+        .chain([(r"\[master \w{7}\]", r"[master COMMIT]")])
+        .collect::<Vec<_>>();
+
+    cmd_snapshot!(filters, commit, @r#"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    [master COMMIT] Update file
+     1 file changed, 1 insertion(+), 1 deletion(-)
+
+    ----- stderr -----
+    echo.....................................................................Passed
+    - hook id: echo
+    - duration: [TIME]
+      file.txt
+    "#);
+
+    Ok(())
+}
+
+/// Test hook id completion works.
+#[cfg(unix)]
+#[test]
+fn completion() {
+    let context = TestContext::new();
+    context.init_project();
+    context.write_pre_commit_config(indoc::indoc! {r"
+        repos:
+          - repo: local
+            hooks:
+              - id: hook
+                name: a useful hook
+                language: system
+                entry: echo
+                verbose: true
+    "});
+    context.git_add(".");
+
+    cmd_snapshot!(context.filters(), context.run().env("COMPLETE", "fish"), @r#"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    complete --keep-order --exclusive --command prek --arguments "(COMPLETE=fish "'[CURRENT_EXE]'" -- (commandline --current-process --tokenize --cut-at-cursor) (commandline --current-token))"
+
+    ----- stderr -----
+    "#);
+
+    cmd_snapshot!(context.filters(), context.run().env("COMPLETE", "fish").arg("--").arg("prek").arg(""), @r#"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    install	Install the prek git hook
+    install-hooks	Create hook environments for all hooks used in the config file
+    run	Run hooks
+    list	List available hooks
+    uninstall	Uninstall the prek git hook
+    validate-config	Validate `.pre-commit-config.yaml` files
+    validate-manifest	Validate `.pre-commit-hooks.yaml` files
+    sample-config	Produce a sample `.pre-commit-config.yaml` file
+    auto-update	Auto-update pre-commit config to the latest repos' versions
+    gc	Clean unused cached repos
+    clean	Clean out pre-commit files
+    init-template-dir	Install hook script in a directory intended for use with `git config init.templateDir`
+    try-repo	Try the pre-commit hooks in the current repo
+    self	`prek` self management
+    hook	a useful hook
+    --all-files	Run on all files in the repo
+    --files	Specific filenames to run hooks on
+    --directory	Run hooks on all files in the specified directories
+    --from-ref	The original ref in a `<from_ref>...<to_ref>` diff expression. Files changed in this diff will be run through the hooks
+    --to-ref	The destination ref in a `from_ref...to_ref` diff expression. Defaults to `HEAD` if `from_ref` is specified
+    --last-commit	Run hooks against the last commit. Equivalent to `--from-ref HEAD~1 --to-ref HEAD`
+    --hook-stage	The stage during which the hook is fired
+    --show-diff-on-failure	When hooks fail, run `git diff` directly afterward
+    --config	Path to alternate config file
+    --color	Whether to use color in output
+    --help	Display the concise help for this command
+    --no-progress	Hide all progress outputs
+    --quiet	Do not print any output
+    --verbose	Use verbose output
+    --version	Display the prek version
+
+    ----- stderr -----
+    "#);
+
+    cmd_snapshot!(context.filters(), context.run().env("COMPLETE", "fish").arg("--").arg("prek").arg("run"), @r#"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    run	Run hooks
+
+    ----- stderr -----
+    "#);
+
+    cmd_snapshot!(context.filters(), context.run().env("COMPLETE", "fish").arg("--").arg("prek").arg("run").arg("h"), @r#"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    hook	a useful hook
+
+    ----- stderr -----
+    "#);
+}
